/*
 * Copyright (C) 2004 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*
 * $Log$
 */

#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include <wx/app.h>
#include <wx/log.h>
#include <wx/listbox.h>
#endif

#include <wx/config.h>
#include <wx/proplist.h>

#include "propertyframe.h"
#include "propertyview.h"
#include "udpthread.h"
#include "mainframe.h"

#include <wx/listimpl.cpp>
WX_DEFINE_LIST(CNutNodeList);


BEGIN_EVENT_TABLE(CMainFrame, wxFrame)
    EVT_LIST_ITEM_ACTIVATED(LIST_CTRL, CMainFrame::OnActivated)
    EVT_MENU(UDP_EVENT, CMainFrame::OnUdpEvent)
END_EVENT_TABLE()

/*!
 * \brief Constructor.
 */
CMainFrame::CMainFrame(const wxString& title)
    : wxFrame(NULL, -1, title)
    , m_nodeList(wxKEY_STRING)
{
    SetIcon(wxICON(nutdisc));

    m_listCtrl = new wxListCtrl(this, LIST_CTRL, wxDefaultPosition, wxDefaultSize,
                                    wxLC_REPORT | wxLC_SINGLE_SEL |
                                    wxSUNKEN_BORDER | wxLC_EDIT_LABELS);

    m_listCtrl->InsertColumn(0, "MAC", wxLIST_FORMAT_LEFT, 100);
    m_listCtrl->InsertColumn(1, "IP", wxLIST_FORMAT_LEFT, 100);
    m_listCtrl->InsertColumn(2, "Mask", wxLIST_FORMAT_LEFT, 100);
    m_listCtrl->InsertColumn(3, "Gate", wxLIST_FORMAT_LEFT, 100);
    m_listCtrl->InsertColumn(4, "Host", wxLIST_FORMAT_LEFT, 100);

    /*
     * Restore frame position and size.
     */
    wxConfigBase *pConfig = wxConfigBase::Get();
    pConfig->SetPath(wxT("/MainFrame"));
    Move(pConfig->Read(wxT("x"), 50), pConfig->Read(wxT("y"), 50));
    SetClientSize(pConfig->Read(wxT("w"), 350), pConfig->Read(wxT("h"), 200));

    m_thread = new CUdpThread(this);
    m_thread->Create();
    m_thread->Run();
}

/*!
 * \brief Destructor.
 */
CMainFrame::~CMainFrame()
{
    m_thread->Kill();

    /* Save window size and position. */
    wxConfigBase *pConfig = wxConfigBase::Get();
    if (pConfig) {
        int x, y;
        GetPosition(&x, &y);
        pConfig->Write(wxT("/MainFrame/x"), (long) x);
        pConfig->Write(wxT("/MainFrame/y"), (long) y);
        GetClientSize(&x, &y);
        pConfig->Write(wxT("/MainFrame/w"), (long) x);
        pConfig->Write(wxT("/MainFrame/h"), (long) y);
    }
}

/*
 * Called when a list entry receives a double click.
 */
void CMainFrame::OnActivated(wxListEvent& event)
{
    long ix = event.GetIndex();

    wxListItem item;
    item.SetId(ix);
    item.SetColumn(0);
    item.SetMask(wxLIST_MASK_TEXT);
    m_listCtrl->GetItem(item);

    wxString mac = item.GetText();
    CNutNodeList::Node *node = m_nodeList.Find(mac);

    if(node) {
        CNutNode *nut = node->GetData();
        CPropertyView *propView = new CPropertyView(nut, m_thread);
        CPropertyFrame *propFrame = new CPropertyFrame(propView, this, nut->m_hostname,
            wxPoint(-1, -1), wxSize(400, 500));
        propFrame->Show(true);
    }
    m_listCtrl->DeleteAllItems();
}

/*
 * Add a new entry to the list.
 */
void CMainFrame::InsertNut(CNutNode *nut)
{
    long ix = m_listCtrl->InsertItem(0, nut->m_mac, 0);

    m_listCtrl->SetItem(ix, 1, nut->m_ip_addr);
    m_listCtrl->SetItem(ix, 2, nut->m_ip_mask);
    m_listCtrl->SetItem(ix, 3, nut->m_gateway);
    m_listCtrl->SetItem(ix, 4, nut->m_hostname);
}

/*
 * Update event from our UDP interface.
 */
void CMainFrame::OnUdpEvent(wxCommandEvent& event)
{
    CNutNode *nut = new CNutNode((DISCOVER *)event.GetInt());

    CNutNodeList::Node *node = m_nodeList.Find(nut->m_mac);
    if(node) {
        m_nodeList.DeleteNode(node);
        m_nodeList.Append(nut->m_mac, nut);
        m_listCtrl->DeleteAllItems();
        for (node = m_nodeList.GetFirst(); node; node = node->GetNext()) {
            InsertNut(node->GetData());
        }
    }
    else {
        m_nodeList.Append(nut->m_mac, nut);
        InsertNut(nut);
    }
    m_listCtrl->SetColumnWidth(0, wxLIST_AUTOSIZE);
    m_listCtrl->SetColumnWidth(1, wxLIST_AUTOSIZE);
    m_listCtrl->SetColumnWidth(2, wxLIST_AUTOSIZE);
    m_listCtrl->SetColumnWidth(3, wxLIST_AUTOSIZE);
    m_listCtrl->SetColumnWidth(4, wxLIST_AUTOSIZE);
}

