/*
 * Copyright (C) 2004 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*
 * $Log$
 */

#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include <wx/app.h>
#include <wx/frame.h>
#include <wx/listbox.h>
#endif

#include "nutnode.h"

CNutNode::CNutNode(DISCOVER *tele)
{
    m_tele = tele;
    m_xid = tele->tele_xid;
    m_hostname = wxString(tele->tele_hostname);
    m_mac = MacToString(tele->tele_mac);
    m_ip_addr = IpToString(tele->tele_ip_addr);
    m_ip_mask = IpToString(tele->tele_ip_mask);
    m_gateway = IpToString(tele->tele_gateway);
    m_cip_addr = IpToString(tele->tele_cip_addr);
}

/*!
 * \brief Destructor.
 */
CNutNode::~CNutNode()
{
    free(m_tele);
}

wxString CNutNode::MacToString(unsigned char mac[6])
{
    wxString str;
    str.Printf("%02X:%02X:%02X:%02X:%02X:%02X", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
    return str;
}

wxString CNutNode::IpToString(unsigned long ip)
{
    unsigned char *cp = (unsigned char *)&ip;
    wxString str;
    str.Printf("%u.%u.%u.%u", cp[0], cp[1], cp[2], cp[3]);
    return str;
}

bool CNutNode::StringToMac(wxString str, unsigned char *mac)
{
    unsigned long val;
    wxString sub;
    int i;

    for(i = 0; i < 6; i++) {
        sub = str.BeforeFirst(':');
        str = str.AfterFirst(':');
        if(!sub.ToULong(&val, 16)) {
            return false;
        }
        mac[i] = (unsigned char)val;
    }
    return true;
}

unsigned long CNutNode::StringToIp(wxString str)
{
    unsigned long ip;
    unsigned char *cp = (unsigned char *)&ip;
    wxString sub;
    unsigned long val;
    int i;

    for(i = 0; i < 4; i++) {
        sub = str.BeforeFirst('.');
        str = str.AfterFirst('.');
        if(!sub.ToULong(&val)) {
            return false;
        }
        cp[i] = (unsigned char)val;
    }
    return ip;
}

